//
//  NIBController.m
//  DynamicNIB
//
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#ifndef INCLUDED_NIBCONTROLLER_H
#import "NIBController.h"
#endif
#ifndef INCLUDED_NIBMANAGER_H
#import "NIBManager.h"
#endif
#ifndef INCLUDED_NIBMANAGERHANDLER_H
#import "NIBManagerHandler.h"
#endif
#ifndef INCLUDED_AZARRAYCONTROLLER_H
#import "AZArrayController.h"
#endif
#ifndef INCLUDED_AZOBJECTCONTROLLER_H
#import "AZObjectController.h"
#endif
#ifndef INCLUDED_AZTREECONTROLLER_H
#import "AZTreeController.h"
#endif
#ifndef INCLUDED_AZVIEW_H
#import "AZView.h"
#endif
#ifndef INCLUDED_OUTLINEVIEWDELEGATE_H
#import "OutlineViewDelegate.h"
#endif



@implementation NIBController


- (id)init {
    self = [super init];
    if (self) {
        controllers = [[[NSMutableArray alloc] init] retain];
        menus = [[[NSMutableDictionary alloc] init] retain];
        manager = nil;
        window = nil;
        model = nil;
    }
    return self;
}


- (void)dealloc {
	if (manager)
		[manager removeController:self];
    if (model)
        [model release];
    if (controllers)
        [controllers release];
    if (menus)
        [menus release];
    if (window)
        [window release];
    [super dealloc];
}


-(void)awakeFromNib {
	if (manager)
		[manager addController:self];
}


- (void)setEditable:(BOOL)editable withContext:(NIBContext *)context forContext:(NSString *)name {
	[self setEditable:editable withContext:context forEntity:name forFilter:name];
}
	
	
- (void)setEditable:(BOOL)editable withContext:(NIBContext *)context forEntity:(NSString *)entity forFilter:(NSString *)filter {
	NSObjectController *	controller;
	NSMutableArray *		array = (filter ? [[context getMetadata] getMembers:filter] : nil);
	NSString *				name;
	int						i;
	
	for (i = 0; i < [[self controllers] count]; i++) {
		controller = [self getController:i];
		name = [controller entityName];
		if ((entity && [entity isEqualTo:name]) || (!filter && (!entity || [entity isEqualTo:name]))) {
			if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
				NSLog(@"NIBController enabling controller: %s [%s]", [name UTF8String], (editable ? "YES" : "NO"));
			[controller setEditable:editable];
		}
	}

	if (!array)
		return;
	if ([array count] == 0)
		[array addObject:filter];
	for (i = 0; i < [array count]; i++)
		[NIBManager setEnabled:editable withView:[self window] withFilter:[array objectAtIndex:i]];
}


- (NSMutableArray *)controllers {
    return controllers;
}


- (void)setControllers:(NSMutableArray *)myControllers {
    if (controllers)
        [controllers release];
    controllers = myControllers;
}


- (void)addController:(NSObjectController *)myController withContext:(NIBContext *)context {
	if ([myController isKindOfClass:[AZObjectController class]]) {
		[(AZObjectController *) myController setCallback:[[self manager] callback]];
		[(AZObjectController *) myController setContext:context];
	}
	if ([myController isKindOfClass:[AZArrayController class]]) {
		[(AZArrayController *) myController setCallback:[[self manager] callback]];
		[(AZArrayController *) myController setContext:context];
	}
	if ([myController isKindOfClass:[AZTreeController class]]) {
		[(AZTreeController *) myController setCallback:[[self manager] callback]];
		[(AZTreeController *) myController setContext:context];
	}
	[[self controllers] addObject:myController];
}


- (NSObjectController *)getController:(int)index {
	return [[self controllers] objectAtIndex:index];
}


- (NSMutableDictionary *)menus {
    return menus;
}


- (void)setMenus:(NSMutableDictionary *)myMenus {
    if (menus)
        [menus release];
    menus = myMenus;
}


- (NSMenu *)getMenu:(NSString *)context {
	return [[self menus] valueForKey:context];
}


- (NIBManager *)manager {
    return manager;
}


- (void)setManager:(NIBManager *)myManager {
    manager = myManager;
}


- (NSView *)window {
    return window;
}


- (void)setWindow:(NSView *)myWindow {
    window = [[self processWindow:[self determineWindow:myWindow]] retain];
}


- (NSView *)determineWindow:(NSView *)myWindow {
/*	NSArray *		views = [myWindow subviews];
	
	if ([views count] == 1) {
		id			temp = [views objectAtIndex:0];
		
		if ([temp isKindOfClass:[NSBox class]] && [[self manager] decorated]) {
			if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
				NSLog(@"NIBController returning decorated NSBox...");
			return temp;
		} else if ([temp isKindOfClass:[NSView class]]) {
			if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
				NSLog(@"NIBController returning embedded NSView...");
			return [self determineWindow:(NSView *)temp];
		}
	}*/
	return myWindow;
}


- (NSView *)processWindow:(NSView *)myWindow {
	if ([[self manager] displayMode] == WINDOWED)
		return myWindow;
		
	if ([[self manager] displayMode] == EMBEDDED) {

		if ([myWindow isKindOfClass:[NSBox class]]) {
			[[(NSBox *)myWindow contentView] setMenu:[myWindow menu]];
			if (![[self manager] decorated]) {
				[(NSBox *)myWindow setTitlePosition:NSNoTitle];
				[(NSBox *)myWindow setBorderType:NSNoBorder];
			}
		}
	
		NSRect *		bounds = [[self manager] bounds];
		NSRect			myBounds = [myWindow bounds];

		if ([[myWindow subviews] count] == 0) {
			[myWindow setFrame:*bounds];
			[myWindow setBounds:*bounds];
			[myWindow setNeedsDisplay:YES];
		} else if ([[self manager] scrolling]) {
			return [NIBManager makeScrollView:myWindow withBounds:(bounds ? *bounds : myBounds) withFlipped:[[self manager] flipped]];
		} else if ([[self manager] flipped]) {
			return [NIBManager makeFlippedView:myWindow];
		}
	}
	return myWindow;
}


- (NSManagedObjectModel *)model {
    return model;
}


- (void)setModel:(NSManagedObjectModel *)myModel {
    model = myModel;
}


- (NSManagedObject *)getObject:(NSString *)context {
	NSObjectController *	controller;
	NSManagedObject *		object;
	NSString *				name;
	int						i = 0;
	
	for (i = 0; i < [[self controllers] count]; i++) {
		controller = [self getController:i];
		name = [controller entityName];
		if (name && (!context || [context isEqualTo:name])) {
			object = [controller content];
			if (object)
				return object;
			if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
				NSLog(@"NIBController creating new NSManagedObject of name: %s", [[controller entityName] UTF8String]);
			[controller setContent:[controller newObject]];
			return [controller content];
		}
	}
	return nil;
}


- (NSMutableDictionary *)doParse:(MetadataParser *)parser {
	return [self doParse:parser withEntity:nil];
}


- (NSMutableDictionary *)doParse:(MetadataParser *)parser stringifyDates:(BOOL)stringify {
	return [self doParse:parser withEntity:nil stringifyDates:stringify];
}


- (NSMutableDictionary *)doParse:(MetadataParser *)parser withEntity:(NSString *)context {
	return [self doParse:parser withEntity:context stringifyDates:YES];
}
	
	
- (NSMutableDictionary *)doParse:(MetadataParser *)parser withEntity:(NSString *)context stringifyDates:(BOOL)stringify {
	NSMutableDictionary *	masterDictionary = [[NSMutableDictionary dictionary] retain];
	id						object;

	if (!context) {
		object = [self getObject:nil];
		if ([object isKindOfClass:[NSManagedObject class]])
			[masterDictionary addEntriesFromDictionary:[self doParse:parser withObject:object stringifyDates:stringify]];
		return masterDictionary;
	}
	
	// The context is a comma seperated list, so tokenize and iterate
	//
	NSArray *				contextList = [context componentsSeparatedByString:@","];
	NSString *				entity;
	int						i;
	
	for (i = 0; i < [contextList count]; i++) {
		entity = [contextList objectAtIndex:i];
		object = [self getObject:entity];
		if ([object isKindOfClass:[NSManagedObject class]]) {
			[masterDictionary addEntriesFromDictionary:[self doParse:parser withObject:object stringifyDates:stringify]];
		} else {
			object = [self getObject:nil];
			if ([object isKindOfClass:[NSManagedObject class]])
				[masterDictionary addEntriesFromDictionary:[self doParse:parser withObject:object stringifyDates:stringify]];
		}
	}
	return masterDictionary;
}


- (NSMutableDictionary *)doParse:(MetadataParser *)parser withObject:(NSManagedObject *)object stringifyDates:(BOOL)stringify {
	NSMutableDictionary *	dictionary = [[NSMutableDictionary dictionary] retain];
	NSDictionary *			properties = [[object entity] propertiesByName];
	NSArray *				keys = [properties allKeys];
	NSString *				key;
	id						value;
	id						temp;
	int		 				i;

	for (i = 0; i < [keys count]; i++) {
		key = [keys objectAtIndex:i];
		temp = [properties objectForKey:key];
		if (temp && [temp isKindOfClass:[NSAttributeDescription class]]) {
			value = [object valueForKey:key];
					
			// Replace list item values with their metadata values if they are specified 
			id component = [NIBManager getComponentForBinding:key fromParent:[self window]];
			if ([component isKindOfClass:[NSPopUpButton class]]) {
				//NSString *listValue = [parser metadataListValueAtIndex:[component indexOfSelectedItem] forIdentifier:key];
				NSString *listValue = [parser metadataListValueAtIndex:[component indexOfSelectedItem] forIdentifier:key forLocale:@"jp"];
				if (listValue) {
					if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
						NSLog(@"NIBController found list value for component: %s, value: %s", [key UTF8String], [listValue UTF8String]);
					value = listValue;
				}
			}
			
			if ([parser isDate:value forKey:key withAttribute:(NSAttributeDescription *)temp]) {
				[dictionary setValue:(stringify ? [parser parseAndFormatDate:value] : value) forKey:key];
			} else if ([parser isDecimalNumber:value forKey:key withAttribute:(NSAttributeDescription *)temp]) {
				[dictionary setValue:(value ? [value stringValue] : @"0") forKey:key];
			} else if ([parser isNumber:value forKey:key withAttribute:(NSAttributeDescription *)temp]) {
				[dictionary setValue:(value ? [value stringValue] : @"0") forKey:key];
			} else if ([parser isString:value forKey:key withAttribute:(NSAttributeDescription *)temp]) {
				[dictionary setValue:(value ? value : @"") forKey:key];
			} else if (value && [NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
				NSLog(@"NIBController unknown datatype for key: %s", [key UTF8String]);
		}
	}
	return dictionary;
}


- (void)doPopulate:(NSDictionary *)data withMetadata:(MetadataParser *)parser {
	[self doPopulate:data withMetadata:parser withEntity:nil];
}


- (void)doPopulate:(NSDictionary *)data withMetadata:(MetadataParser *)parser withEntity:(NSString *)context {
	[self doPopulate:data withMetadata:parser withObject:[self getObject:context]];
}

- (void)doPopulateForKey:(NSString*)key withData:(NSDictionary *)data withMetadata:(MetadataParser *)parser withObject:(NSManagedObject *)object {
	NSDictionary *			properties = [[object entity] propertiesByName];
	id						value;
	id						temp;
	
	temp = [properties objectForKey:key];
	if (temp && [temp isKindOfClass:[NSAttributeDescription class]]) {
		value = [data objectForKey:key];
			
		// Use the metadata list item values, if available, to set the correct selection based on the value 
		id component = [NIBManager getComponentForBinding:key fromParent:[self window]];
		if ([component isKindOfClass:[NSPopUpButton class]]) {
			NSArray *listValues = [parser metadataListValues:key forLocale:@"jp"];
			if (listValues && [listValues indexOfObject:value] != NSNotFound) {
				value = [component itemTitleAtIndex:[listValues indexOfObject:value]];
				if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
					NSLog(@"NIBController found list title for component: %s, value: %s", [key UTF8String], [value UTF8String]);
			}
				
			// If this is a Dynamic PopUp, fire the parent change even before populating
			if ([NIBController isDynamicPopUp:key withMetadata:parser]) {
				NSString *parentWidgetName = [[parser getValue:key] substringFromIndex:9];
				parentWidgetName = [parentWidgetName substringToIndex:[parentWidgetName length]-1];
				id parentWidget = [NIBManager getComponentForBinding:parentWidgetName fromParent:[self window]];
				if (parentWidget && [parentWidget isKindOfClass:[NSPopUpButton class]]) {
					[[[self manager] handler] onPopUpChange:parentWidget];
				}
			}
		}
		if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
			NSLog(@"NIBController assigning object: %s = %s", [key UTF8String], [[value description] UTF8String]);
		[object setValue:[parser formatObject:value forKey:key withAttribute:(NSAttributeDescription *)temp] forKey:key];
	}
}

- (void)doPopulate:(NSDictionary *)data withMetadata:(MetadataParser *)parser withObject:(NSManagedObject *)object {
	NSArray *				keys = [data allKeys];
	NSArray *				subviews = [NIBManager getComponents:[self window]];
	NSMutableArray *		childrenPopUpKeys = [[NSMutableArray alloc] init];
	NSString *				key;
	id						temp;
	int		 				i;

	// Process keys
	for (i = 0; i < [keys count]; i++) {
		key = [keys objectAtIndex:i];
		
		// If this is a Dynamic PopUp, add to the list and process at the end
		if([NIBController isDynamicPopUp:key withMetadata:parser]) {
			[childrenPopUpKeys addObject:key];
			//[self doPopulateForKey:key withData:data withMetadata:parser withObject:object];
		}
		else
			[self doPopulateForKey:key withData:data withMetadata:parser withObject:object];
	}
	
	// Process DynamicPopUp keys
	for (i = 0; i < [childrenPopUpKeys count]; i++) {
		key = [childrenPopUpKeys objectAtIndex:i];
		[self doPopulateForKey:key withData:data withMetadata:parser withObject:object];
	}
	
	// Process Subviews
	for (i = 0; i < [subviews count]; i++) {
		temp = [subviews objectAtIndex:i];
		if ([temp isKindOfClass:[NSOutlineView class]] && [[temp delegate] isKindOfClass:[OutlineViewDelegate class]])
			[[temp delegate] doPopulate];
	}
}

+ (BOOL)isDynamicPopUp:(NSString*)key withMetadata:(MetadataParser *)parser {
	NSString *drop = [parser getValue:key];
	return [drop hasPrefix:@"<DYNAMIC_"];
}

- (BOOL)doMerge:(NIBController *)myController {
	NSObjectController *	controller;
	NSObjectController *	temp;
	int						i, j;
	BOOL					found = NO;
	
	for (i = 0; i < [[self controllers] count]; i++) {
		controller = [self getController:i];
		if ([controller entityName])
			for (j = 0; j < [[myController controllers] count]; j++) {
				temp = [myController getController:j];
				if ([[controller entityName] isEqualTo:[temp entityName]]) {
					if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
						NSLog(@"NIBController merging NSObjectController entity: %s", [[controller entityName] UTF8String]);
					if ([temp isKindOfClass:[NSArrayController class]] && [controller isKindOfClass:[NSArrayController class]]) {
						[(NSArrayController *)controller removeObjects:[(NSArrayController *)controller arrangedObjects]];
						[(NSArrayController *)controller addObjects:[(NSArrayController *)temp arrangedObjects]];
						found = YES;
					} else if ([temp isKindOfClass:[NSTreeController class]] && [controller isKindOfClass:[NSTreeController class]]) {
						NSMutableArray *tempContent = [[NSMutableArray alloc] init];
						[tempContent addObjectsFromArray: [(NSTreeController *)temp content]];
						[(NSTreeController *)controller setContent:tempContent];
						found = YES;
					} else if ([[temp content] isKindOfClass:[NSManagedObject class]] && [[controller content] isKindOfClass:[NSManagedObject class]]) {
						[NIBController doMerge:[temp content] toObject:[controller content]];
						found = YES;
					}
				}
			}
	}
	return NO;
}


+ (BOOL)doMerge:(NSManagedObject *)source toObject:(NSManagedObject *)destination {
	NSDictionary *			prop1 = [[source entity] propertiesByName];
	NSDictionary *			prop2 = [[destination entity] propertiesByName];
	NSArray *				keys1 = [prop1 allKeys];
	NSArray *				keys2 = [prop2 allKeys];
	NSString *				key;
	id						temp;
	int		 				i;

	for (i = 0; i < [keys1 count]; i++) {
		key = [keys1 objectAtIndex:i];
		temp = [prop1 objectForKey:key];
		if (temp && [temp isKindOfClass:[NSAttributeDescription class]] && [keys2 containsObject:key]) {
			[destination setValue:[source valueForKey:key] forKey:key];
		}
	}
	return YES;
}


- (IBAction)doExecute:(id)sender {
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
        NSLog(@"NIBController received execute event...");
    [self doCallback: sender];
}


- (IBAction)doCallback:(id)sender {
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
        NSLog(@"NIBController received callback event...");
    if ([self isValid:nil]) {
		int		i = 0; 
		
		for (i = 0; i < [[self controllers] count]; i++)
			[[self manager] doCallback:self withObject:[self getController:i]];
	}
}


- (void)doValidate {
	int		i;
	
	for (i = 0; i < [[self controllers] count]; i++)
		[[self getController:i] commitEditing];
}


- (void)doSetup {
	NSObjectController *	controller;
	int						i = 0;
	
	for (i = 0; i < [[self controllers] count]; i++) {
		controller = [self getController:i];
		if ([controller entityName] && ![controller isKindOfClass:[NSArrayController class]]) {
			if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
				NSLog(@"NIBController creating new NSManagedObject of name: %s", [[controller entityName] UTF8String]);
			[controller setContent:[controller newObject]];
		}
	}
}


- (void)doInitialise {
	[self doSetup];
}


- (void)doInitialiseWith:(NIBManager *)myManager {
    NSString *                      directory = nil;
	NSString *						store = [[myManager backingStore] lastPathComponent];
	NSArray *                       paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES);
	NSMutableArray *				path = [NSMutableArray arrayWithArray:[[myManager backingStore] pathComponents]];
    NSFileManager *                 fileManager = [NSFileManager defaultManager];
    BOOL                            isDirectory = NO;
    
	[path removeLastObject];
    if ([paths count] == 1) {
        directory = [[[paths objectAtIndex:0] stringByAppendingPathComponent:[path componentsJoinedByString:@"/"]] retain];
        if (![fileManager fileExistsAtPath:directory isDirectory:&isDirectory]) {
		
			NSString* dir = [[paths objectAtIndex:0] retain];
			int i;
			for(i = 0 ; i < [path count] ; i++) {
				NSString* pathElement = [path objectAtIndex:i];
				dir = [dir stringByAppendingPathComponent:pathElement];
				if (![fileManager fileExistsAtPath:dir isDirectory:&isDirectory]) {
					if (![fileManager createDirectoryAtPath:dir attributes:nil]) {
						[dir release];
						dir = nil;
						return;
					}
				}
			}
		} else if (!isDirectory) {
            [directory release];
            directory = nil;
        }
    }

    id                              newStore;
    NSError *                       error;
    NSURL *                         url = [NSURL fileURLWithPath: [directory stringByAppendingPathComponent:store]];
    NSPersistentStoreCoordinator *  coordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel: model]; 
	NSObjectController *			controller;
	int								i;
	
	for (i = 0; i < [[self controllers] count]; i++) {
		controller = [self getController:i];
		if ([controller entityName]) {
			if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
				NSLog(@"NIBController configuring CoreData coordinator/store for: %s", [[controller entityName] UTF8String]);
			[[controller managedObjectContext] setPersistentStoreCoordinator: coordinator];
			if (newStore == nil)
				newStore = [coordinator addPersistentStoreWithType:[manager backingType]
													 configuration:nil
															   URL:url
														   options:nil
															 error:&error];
			if (newStore == nil)
				NSLog(@"NIBController store configuration failure: %@", ([error localizedDescription] != nil) ? [error localizedDescription] : @"Unknown Error");
		}
	}
    [coordinator release];
}


- (BOOL)isValid:(NSError **)error {
	NSObjectController *			controller;
	BOOL							valid = YES;
	int								i = 0;
	
	for (i = 0; i < [[self controllers] count]; i++) {
		controller = [self getController:i];
		if ([[controller content] isKindOfClass:[NSManagedObject class]])
			valid = [[controller content] validateForUpdate:error];
		if (!valid)
			return NO;
	}
	return valid;
}


- (BOOL)isValid:(NSError **)error forEntity:(NSString *)context {
    id				object = [self getObject:context];
	return ([object isKindOfClass:[NSManagedObject class]] ? [object validateForUpdate:error] : NO);
}


- (BOOL)isEnabled {
    return ([[self controllers] count] > 0);
}


@end
