#ifndef INCLUDED_NIBMANAGER_H
#define INCLUDED_NIBMANAGER_H
//
//  NIBManager.h
//  DynamicNIB
//
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#ifndef INCLUDED_NIBCONTROLLER_H
#import "NIBController.h"
#endif
#ifndef INCLUDED_NIBCALLBACK_H
#import "NIBCallback.h"
#endif
#ifndef INCLUDED_NIBCONTEXT_H
#import "NIBContext.h"
#endif
#ifndef INCLUDED_NIBVALIDATOR_H
#import "NIBValidator.h"
#endif
#ifndef INCLUDED_METADATAPARSER_H
#import "MetadataParser.h"
#endif
#ifndef INCLUDED_DYNAMICPOPUPHANDLER_H
#import "DynamicPopUpHandler.h"
#endif



extern NSString *	DEFAULT_BACKING_STORE;
extern NSString *	DEFAULT_BACKING_PATH; 
extern NSString *	DEFAULT_NIB_NAME; 


@interface NIBManager : NSObject {

    
	id						main;
	DynamicPopUpHandler *	handler;
    NSString *          path;
    NSString *          name;
    NSString *          backingStore;
    NSString *          backingType;
    NSMutableArray *    contexts;
    NSMutableArray *    controllers;
    NIBCallback *       callback;
    NIBValidator *		validator;
    NSTabView *         tabView;
    NSPopUpButton *     popupMenu;
	NSRect *			bounds;
	NSPoint *			adjustment;
	int					displayMode;
	int					persistenceMode;
    BOOL				scrolling;
    BOOL				flipped;
    BOOL				decorated;
    BOOL				assumeAll;
}


- (NSArray *)topLevelObjectsFromLoadingNibNamed:(NSString *)myName withOwner:(id)owner;
- (NSArray *)topLevelObjectsFromLoadingBundle:(NSBundle *)bundle withName:(NSString *)myName withOwner:(id)owner;
- (NSArray *)topLevelObjectsFromLoadingBundleNib:(NSBundle *)bundle withName:(NSString *)myName withOwner:(id)owner;


- (int)displayMode;
- (int)persistenceMode;
- (BOOL)scrolling;
- (BOOL)flipped;
- (BOOL)decorated;
- (BOOL)assumeAll;
- (NSRect *)bounds;
- (NSPoint *)adjustment;
- (DynamicPopUpHandler *)handler;
- (void)setScrolling:(BOOL)isScrolling;
- (void)setFlipped:(BOOL)isFlipped;
- (void)setDecorated:(BOOL)isDecorated;
- (void)setAssumeAll:(BOOL)isAssumed;
- (void)setDisplayMode:(int)myMode;
- (void)setPersistenceMode:(int)myMode;
- (void)setBounds:(NSRect)myBounds;
- (void)setAdjustment:(NSPoint)myAdjustment;


- (id)main;
- (NSString *)path;
- (NSString *)name;
- (NSString *)backingStore;
- (NSString *)backingType;
- (NSTabView *)tabView;
- (NSPopUpButton *)popupMenu;
- (NIBCallback *)callback;
- (NIBValidator *)validator;
- (void)setMain:(id)myMain;
- (void)setPath:(NSString *)myPath;
- (void)setName:(NSString *)myName;
- (void)setBackingStore:(NSString *)myStore;
- (void)setBackingType:(NSString *)myType;
- (void)setTabView:(NSTabView *)myTabs;
- (void)setCallback:(NIBCallback *)myCallback;
- (void)setValidator:(NIBValidator *)myValidator;
- (void)setHandler:(DynamicPopUpHandler *)myHandler;
- (void)setPopupMenu:(NSPopUpButton *)myMenu;
- (void)setPopupMenu:(NSPopUpButton *)myMenu withTarget:(id)target withAction:(SEL)action;

- (NIBContext *)getContext:(int)index;
- (NIBContext *)getContextFor:(NSString *)type;

- (NIBController *)getController:(int)index;
- (NIBController *)createController;
- (void)addController:(NIBController *)controller;
- (void)removeController:(NIBController *)controller;
- (int)getContextCount;
- (int)getControllerCount;
- (int)determineTag:(NSString *)type;

- (IBAction)onPopUpChange:(id)sender;
- (IBAction)doPopUpChange:(id)sender;
- (IBAction)doExecute:(id)sender;
- (void)doDisplay:(NSString *)type;
- (void)doDisplay:(NSString *)type withIdentifier:(id)identifier;
- (void)doInitialise:(NSString *)myPath;
- (void)doInitialise:(NSString *)myPath withCallback:(NIBCallback *)myCallback;
- (void)doInitialise:(NSString *)myPath withCallback:(NIBCallback *)myCallback withValidator:(NIBValidator *)myValidator;
- (void)doBuild:(NSPopUpButton *)menu;
- (void)doBuild:(NSPopUpButton *)menu withTarget:(id)target withAction:(SEL)action;
- (void)doBuild:(NSPopUpButton *)menu withTabs:(NSTabView *)tabs;
- (void)doBuild:(NSPopUpButton *)menu withTabs:(NSTabView *)tabs withTarget:(id)target withAction:(SEL)action;
- (void)doBuildSearch:(NSSearchField *)field withTemplate:(NSMenu *)menu withSelector:(SEL)aSelector;
- (void)doCallback:(NIBController *)controller withObject:(id)object;
- (NIBContext *)doCreate:(NSString *)myPath withName:(NSString *)myName withTag:(int)tag;
- (BOOL)isEnabled;

- (NSArray *)createResultsArrayForDisplay:(NSArray *)results;
- (NSDictionary *)createResultsDictionaryForDisplay:(NSDictionary *)results;

- (void)processDecorations:(NSView *)aView forContext:(NIBContext *)context;
- (void)processTextDecorations:(id)component forContext:(NIBContext *)context forBinding:(NSString *)binding withParent:(NSView *)parent;
- (void)processTokenDecorations:(id)component forContext:(NIBContext *)context forBinding:(NSString *)binding withParent:(NSView *)parent;
- (void)processPopUpDecorations:(id)component forContext:(NIBContext *)context forBinding:(NSString *)binding withParent:(NSView *)parent;
- (void)processOutlineDecorations:(id)component forContext:(NIBContext *)context forBinding:(NSString *)binding withParent:(NSView *)parent;
- (void)processValidationDecorations:(id)component forContext:(NIBContext *)context forBinding:(NSString *)binding withParent:(NSView *)parent;
- (void)processDatePickerDecorations:(id)component forContext:(NIBContext *)context forBinding:(NSString *)bindingName withParent:(NSView *)parent;

+(void)setEnabled:(BOOL)enabled withView:(NSView *)view;
+(void)setEnabled:(BOOL)enabled withView:(NSView *)view withFilter:(NSString *)filter;
+(void)setValue:(id)value withKey:(NSString *)key withParent:(NSView *)view;
+(void)removeComponent:(NSView *)view;
+(void)removeComponent:(NSView *)view fromParent:(NSView *)parent;
+(void)addComponent:(NSView *)view withTabs:(NSTabView *)tabs withIdentifier:(NSString *)identifier;
+(void)addComponent:(NSView *)view withTabs:(NSTabView *)tabs withItem:(NSTabViewItem *)item;
+(void)addAction:(NSView *)view withTarget:(id)target withAction:(SEL)action;
+(void)addAction:(NSView *)view withDelegate:(id)delegate withTarget:(id)target withAction:(SEL)action;
+(void)originateComponent:(NSView *)component;
+(id)getContentComponent:(id)component;
+(id)getComponent:(NSString *)key fromParent:(id)component;
+(id)getComponentOrView:(NSString *)key fromParent:(id)component;
+(id)getComponentForBinding:(NSString *)bindingName fromParent:(id)component;
+(NSMutableArray *)getComponents:(id)parent;
+(id)determineComponent:(id)component;
+(NSString *)determineName:(id)component;
+(NSString *)determineIdentifier:(id)component;
+(NSString *)determineEnabled:(id)component;
+(NSString *)determineBinding:(NSString *)key withComponent:(id)component;
+(NSString *)determineBinding:(NSString *)key withComponent:(id)component withOffset:(int)offset;
+(NSView *)makeFlippedView:(NSView *)component;
+(NSView *)makeFlippedView:(NSView *)component withAddition:(BOOL)addition;
+(NSBox *)makeBoxedView:(NSView *)component withTitle:(NSString *)title;
+(NSBox *)makeBoxedView:(NSView *)component withTitle:(NSString *)title withFlipped:(BOOL)flipped;
+(NSScrollView *)makeScrollView:(NSView *)component withFlipped:(BOOL)flipped;
+(NSScrollView *)makeScrollView:(NSView *)component withBounds:(NSRect)bounds withFlipped:(BOOL)flipped;
+(BOOL)makeDirectoryPath:(NSString *)path;

+(NSWindow *)makeWindow:(NSView *)component withTitle:(NSString *)title;
+(NSWindow *)makeWindow:(NSView *)component withTitle:(NSString *)title withStyle:(unsigned int)style;
+(NSPanel *)makePanel:(NSView *)component withTitle:(NSString *)title;
+(NSPanel *)makePanel:(NSView *)component withTitle:(NSString *)title withStyle:(unsigned int)style;

+(NSWindow *)makeWindow:(NIBContext *)context withView:(NSView *)view withTitle:(NSString *)title;
+(NSWindow *)makeWindow:(NIBContext *)context withView:(NSView *)view withTitle:(NSString *)title withStyle:(unsigned int)style;
+(NSPanel *)makePanel:(NIBContext *)context withView:(NSView *)view withTitle:(NSString *)title;
+(NSPanel *)makePanel:(NIBContext *)context withView:(NSView *)view withTitle:(NSString *)title withStyle:(unsigned int)style;


@end
#endif /* defined INCLUDED_NIBMANAGER_H */
