//
//  NIBManager.m
//  DynamicNIB
//
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#ifndef INCLUDED_SM2DGRAPHVIEW_H
#import <SM2DGraphView/SM2DGraphView.h>
#endif
#ifndef INCLUDED_SM2DPIECHARTVIEW_H
#import <SM2DGraphView/SMPieChartView.h>
#endif
#ifndef INCLUDED_NIBMANAGER_H
#import "NIBManager.h"
#endif
#ifndef INCLUDED_NIBMANAGERHANDLER_H
#import "NIBManagerHandler.h"
#endif
#ifndef INCLUDED_NIBCONTROLLER_H
#import "NIBController.h"
#endif
#ifndef INCLUDED_NIBCALLBACK_H
#import "NIBCallback.h"
#endif
#ifndef INCLUDED_AZWINDOW_H
#import "AZWindow.h"
#endif
#ifndef INCLUDED_AZTEXTFIELD_H
#import "AZTextField.h"
#endif
#ifndef INCLUDED_AZVIEW_H
#import "AZView.h"
#endif
#ifndef INCLUDED_AZSCROLLVIEW_H
#import "AZScrollView.h"
#endif
#ifndef INCLUDED_METADATAPARSER_H
#import "MetadataParser.h"
#endif
#ifndef INCLUDED_DECORATORPARSER_H
#import "DecoratorParser.h"
#endif
#ifndef INCLUDED_MATRIXCONTROLLER_H
#import "MatrixController.h"
#endif
#ifndef INCLUDED_DYNAMICPOPUPHANDLER_H
#import "DynamicPopUpHandler.h"
#endif
#ifndef INCLUDED_OUTLINEVIEWCONTROLLER_H
#import "OutlineViewController.h"
#endif
#ifndef INCLUDED_OUTLINEVIEWMANAGER_H
#import "OutlineViewManager.h"
#endif
#ifndef INCLUDED_OUTLINEVIEWDELEGATE_H
#import "OutlineViewDelegate.h"
#endif
#ifndef INCLUDED_TOKENFIELDDELEGATE_H
#import "TokenFieldDelegate.h"
#endif
#ifndef INCLUDED_PADDEDFORMATTER_H
#import "PaddedFormatter.h"
#endif


NSString *	DEFAULT_BACKING_STORE		= @"Logs/DocHandler/DocHandler.xobj";
NSString *	DEFAULT_NIB_NAME			= @"AozoraBT";
NSString *  DEFAULT_RESULT_TYPE_KEY     = @"r_object_type";
NSString *  DEFAULT_RESULT_DATA_KEY     = @"data";


float			XADJUST				= 20;
float			YADJUST				= 20;
BOOL			PROCESS_DECORATIONS	= YES;


@implementation NIBManager


- (id)init {
    self = [super init];
    if (self) {
        main = nil;
        path = nil;
        tabView = nil;
        popupMenu = nil;
        callback = nil;
        validator = nil;
        bounds = nil;	
        adjustment = nil;
		scrolling = YES;
		flipped = YES;
		decorated = NO;
		assumeAll = NO;
		displayMode = EMBEDDED;
		persistenceMode = DOCUMENT;
        name = DEFAULT_NIB_NAME;
        backingType = NSXMLStoreType;
        backingStore = DEFAULT_BACKING_STORE;
        contexts = [[[NSMutableArray alloc] init] retain];
        controllers = [[[NSMutableArray alloc] init] retain];
        handler = [[[DynamicPopUpHandler alloc] init] retain];

		NSPoint myPoint = {XADJUST, YADJUST};
		[self setAdjustment:myPoint];
    }
    return self;
}


- (void)dealloc {
    if (path)
        [path release];
    if (name)
        [name release];
    if (backingType)
        [backingType release];
    if (backingStore)
        [backingStore release];
    if (callback)
        [callback release];
    if (validator)
        [validator release];
    if (contexts)
        [contexts release];
    if (controllers)
        [controllers release];
    if (handler)
        [handler release];
	if (bounds)
		free(bounds);
	if (adjustment)
		free(adjustment);
    [super dealloc];
}


-(void)awakeFromNib {
}


- (NSString *)name {
    return name;
}


- (void)setName:(NSString *)myName {
    if (name)
        [name release];
    name = [[myName stringByExpandingTildeInPath] retain];
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_NOTICE)
        NSLog(@"NIBManager assigning bundle name: %s", [name UTF8String]);
}


- (NSString *)path {
    return path;
}


- (void)setPath:(NSString *)myPath {
    if (path)
        [path release];
    path = [[myPath stringByExpandingTildeInPath] retain];
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_NOTICE)
        NSLog(@"NIBManager assigning bundle path: %s", [path UTF8String]);
}


- (NSString *)backingStore {
    return backingStore;
}


- (void)setBackingStore:(NSString *)myStore {
    if (backingStore)
        [backingStore release];
    backingStore = [[myStore stringByExpandingTildeInPath] retain];
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_NOTICE)
        NSLog(@"NIBManager assigning CoreData backing store: %s", [backingStore UTF8String]);
}


- (NSString *)backingType {
    return backingType;
}


- (void)setBackingType:(NSString *)myType {
    if (backingType)
        [backingType release];
    backingType = myType;
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_NOTICE)
        NSLog(@"NIBManager assigning CoreData backing type: %s", [backingType UTF8String]);
}


- (NSTabView *)tabView {
    return tabView;
}


- (void)setTabView:(NSTabView *)myTabs {
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
        NSLog(@"NIBManager registering NSTabView for displays");
    tabView = myTabs;
}


- (NSPopUpButton *)popupMenu {
    return popupMenu;
}


- (void)setPopupMenu:(NSPopUpButton *)myMenu {
	[self setPopupMenu:myMenu withTarget:self withAction:@selector(onPopUpChange:)];
}


- (void)setPopupMenu:(NSPopUpButton *)myMenu withTarget:(id)target withAction:(SEL)action {
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
        NSLog(@"NIBManager registering NSPopUpButton for changes");
    popupMenu = myMenu;
    if ([self isEnabled]) {
        [popupMenu setTarget:target];
        [popupMenu setAction:action];
    }
}


- (NIBCallback *)callback {
    return callback;
}


- (void)setCallback:(NIBCallback *)myCallback {
    if (myCallback && [NIBManagerHandler getLoggingLevel] >= LOGGING_NOTICE)
        NSLog(@"NIBManager assigning callback: %s", [[myCallback description] UTF8String]);
    callback = myCallback;
}


- (NIBValidator *)validator {
    return validator;
}


- (void)setValidator:(NIBValidator *)myValidator {
    if (myValidator && [NIBManagerHandler getLoggingLevel] >= LOGGING_NOTICE)
        NSLog(@"NIBManager assigning validator: %s", [[myValidator description] UTF8String]);
    validator = myValidator;
}


- (id)main {
    return main;
}


- (void)setMain:(id)myMain {
    if (myMain && [NIBManagerHandler getLoggingLevel] >= LOGGING_NOTICE)
        NSLog(@"NIBManager assigning main: %s", [[myMain description] UTF8String]);
    main = myMain;
}


- (DynamicPopUpHandler *)handler {
    return handler;
}


- (void)setHandler:(DynamicPopUpHandler *)myHandler {
    if (myHandler && [NIBManagerHandler getLoggingLevel] >= LOGGING_NOTICE)
        NSLog(@"NIBManager assigning popup handler: %s", [[myHandler description] UTF8String]);
	if (handler)
		[handler release];
    handler = myHandler;
}


-(int)displayMode {
	return displayMode;
}


-(void)setDisplayMode:(int)myMode {
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
        NSLog(@"NIBManager assigning display mode: %s", (myMode == EMBEDDED ? "EMBEDDED" : "WINDOWED"));
	displayMode = myMode;
}


-(int)persistenceMode {
	return persistenceMode;
}


-(void)setPersistenceMode:(int)myMode {
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
        NSLog(@"NIBManager assigning persistence mode: %s", (myMode == DOCUMENT ? "DOCUMENT" : "UNIQUE"));
	persistenceMode = myMode;
}


-(NSRect *)bounds {
	return bounds;
}


-(void)setBounds:(NSRect)myBounds {
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
        NSLog(@"NIBManager assigning component bounds: [%fx%f] at [%f,%f]", myBounds.size.width, myBounds.size.height, myBounds.origin.x, myBounds.origin.y);
	if (!bounds)
		bounds = (NSRect *)malloc(sizeof(NSRect));
	if (!bounds)
        NSLog(@"NIBManager: out of memory");
	else
		memcpy(bounds, &myBounds, sizeof(NSRect)); 
}


-(NSPoint *)adjustment {
	return adjustment;
}


-(void)setAdjustment:(NSPoint)myAdjustment {
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
        NSLog(@"NIBManager assigning component adjust: at [%f,%f]", myAdjustment.x, myAdjustment.y);
	if (!adjustment)
		adjustment = (NSPoint *)malloc(sizeof(NSPoint));
	if (!adjustment)
        NSLog(@"NIBManager: out of memory");
	else {	
		adjustment->x = myAdjustment.x;
		adjustment->y = myAdjustment.y;	
	}
}


-(BOOL)scrolling {
    return scrolling;
}


-(void)setScrolling:(BOOL)isScrolling {
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
        NSLog(@"NIBManager assigning scrolling views: %s", (isScrolling ? "YES" : "NO"));
    scrolling = isScrolling;
}


-(BOOL)decorated {
    return decorated;
}


-(void)setDecorated:(BOOL)isDecorated {
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
        NSLog(@"NIBManager assigning decorating views: %s", (isDecorated ? "YES" : "NO"));
    decorated = isDecorated;
}


-(BOOL)flipped {
    return flipped;
}


-(void)setFlipped:(BOOL)isFlipped {
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
        NSLog(@"NIBManager assigning flipped views: %s", (isFlipped ? "YES" : "NO"));
    flipped = isFlipped;
}


-(BOOL)assumeAll {
    return assumeAll;
}


-(void)setAssumeAll:(BOOL)isAssumed {
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
        NSLog(@"NIBManager assigning assuming all entities: %s", (isAssumed ? "YES" : "NO"));
    assumeAll = isAssumed;
}


- (void)doInitialise:(NSString *)myPath withCallback:(NIBCallback *)myCallback withValidator:(NIBValidator *)myValidator {
    [self setValidator:myValidator];
    [self doInitialise:myPath withCallback:myCallback];
}


- (void)doInitialise:(NSString *)myPath withCallback:(NIBCallback *)myCallback {
    [self setCallback:myCallback];
    [self doInitialise:myPath];
}


- (void)doInitialise:(NSString *)myPath {
    NSArray *			paths;
	NIBContext *		context;
    NSString *          actual;
	NSString *			temp;
    int                 i;
	int					count = 0;
    
    [self setPath: myPath];
	paths = [[[[NSFileManager defaultManager] directoryContentsAtPath:[self path]] retain] sortedArrayUsingSelector:@selector(localizedCompare:)];
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
        NSLog(@"NIBManager assigning bundle count: %i", [paths count]);
    [contexts removeAllObjects];
    for (i = 0; i < [paths count]; i++) {
		temp = (NSString *) [paths objectAtIndex: i];
        actual = [[[self path] stringByAppendingString:@"/"] stringByAppendingString:temp];
        if ([actual hasSuffix:@".bundle"]) {
			context = [self doCreate:actual withName:temp withTag:count];
			if (context) {
				[contexts addObject:context];
				count++;
			}
        }
    }
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
        NSLog(@"NIBManager initialisation completed");
}


- (NIBContext *)doCreate:(NSString *)myPath withName:(NSString *)myName withTag:(int)tag {
    NIBController *     controller;
	NIBContext *		context = [[[NIBContext alloc] init] retain];
    NSBundle *          bundle;
    NSArray *           objects;
    NSObject *          object;
	NSString *			identifier;
    BOOL                found;
    int                 j;
    
	[context setPath:myPath];
	[context setName:myName];
	if ([NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
		NSLog(@"NIBManager loading bundle at path: %s", [myPath UTF8String]);
		
	bundle = [NSBundle bundleWithPath:myPath];
	if (bundle) {
		if ([NIBManagerHandler getLoggingLevel] >= LOGGING_NOTICE)
			NSLog(@"NIBManager loading DynamicNIB: %s, bundle version: %s", [myName UTF8String], [[[bundle infoDictionary] objectForKey:@"CFBundleVersion"] UTF8String]);
		found = NO;
		[context setBundle:bundle];
		[context setMetadata:[MetadataParser metadataWithBundle:bundle]];
		objects = [self topLevelObjectsFromLoadingBundle:bundle withName:myName withOwner:self];
		if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
			NSLog(@"NIBManager found top-level objects: %i", [objects count]);
		for (j = 0; !found && j < [objects count]; j++) {
			object = [objects objectAtIndex:j];
			if ([object isKindOfClass:[NIBController class]]) {
				controller = (NIBController *) object;
				[controller setModel: [NSManagedObjectModel mergedModelFromBundles:[NSArray arrayWithObject:bundle]]];
				[controller setManager: self];
				found = YES;
			}
		}
		if (!found) {
			controller = [self createController];
			[controller setModel:[NSManagedObjectModel mergedModelFromBundles:[NSArray arrayWithObject:bundle]]];
			[controller setManager: self];
			[self addController: controller];
			for (j = 0; j < [objects count]; j++) {
				object = [objects objectAtIndex:j];
				identifier = [NIBManager determineIdentifier:object];
				
				if ([object isKindOfClass:[NSPanel class]])
					[object retain];
				if ([object isKindOfClass:[OutlineViewController class]]) {
					[(OutlineViewController *)object setPath:[OutlineViewController determinePath:context]];
					[(OutlineViewController *)object doInitialise:self];
					[object retain];
				}

				if (identifier && [object isKindOfClass:[NSMenu class]]) {
					[[controller menus] setValue:(NSMenu *)object forKey:identifier];
				} else if ([object isKindOfClass:[NSView class]]) {
					[controller setWindow:(NSView *)object];
				} else if ([object isKindOfClass:[NSObjectController class]]) {
					[(NSObjectController *)object setManagedObjectContext:[[NSManagedObjectContext alloc] init]];
					[controller addController:(NSObjectController *)object withContext:context];
				}
			}
		}
		[controller doInitialiseWith:self];
		[controller doInitialise];
		[context setController:controller];
		[[context getMetadata] setTag:tag];
		[self processDecorations:[context getView] forContext:context];
		return context;
	} else if ([NIBManagerHandler getLoggingLevel] >= LOGGING_NOTICE) {
		NSLog(@"NIBManager could not load bundle: %s", [myPath UTF8String]);
	}
	return nil;
}


- (void)processDecorations:(NSView *)aView forContext:(NIBContext *)context {
	if (!PROCESS_DECORATIONS)
		return;
		
   	NSArray *       subViews = [NIBManager getComponents:aView];
    NSString *      bindingName;
    id				component;
    int             i;
    
    for (i = 0; i < [subViews count]; i++) {
        component = [subViews objectAtIndex:i];
        if ([component isKindOfClass:[NSControl class]]) {
            bindingName = [NIBManager determineName:component];
            if (bindingName && [bindingName length] > 0) {
                if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
                    NSLog(@"NIBManager checking for decorations for component: %s", [bindingName UTF8String]);
				[self processTextDecorations:component forContext:context forBinding:bindingName withParent:aView];
				[self processPopUpDecorations:component forContext:context forBinding:bindingName withParent:aView];
				[self processValidationDecorations:component forContext:context forBinding:bindingName withParent:aView];
				[self processTokenDecorations:component forContext:context forBinding:bindingName withParent:aView];
				[self processDatePickerDecorations:component forContext:context forBinding:bindingName withParent:aView];
            }
			[self processOutlineDecorations:component forContext:context forBinding:bindingName withParent:aView];
        } else if ([component isKindOfClass:[NSView class]] && ![component isEqual:aView]) {
            [self processDecorations:(NSView *)component forContext:context];
        }        
    }    
}


- (void)processTextDecorations:(id)component forContext:(NIBContext *)context forBinding:(NSString *)bindingName withParent:(NSView *)parent {
	if (![component isKindOfClass:[NSTextField class]])
		return;

	MetadataParser *	parser = [context getMetadata];
	NSString *			type = [parser getElementType:bindingName];

	if ([type isEqualTo:TYPE_PADDED]) {
		if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
			NSLog(@"NIBManager adding PaddedFormatter to attribute: %s", [bindingName UTF8String]);
		[component setFormatter:[PaddedFormatter formatterWithLength:[parser getElementLength:bindingName]]];
	}
	if ([type isEqualTo:TYPE_VALIDATOR] && [self validator] && [component isKindOfClass:[AZTextField class]]) {
		if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
			NSLog(@"NIBManager adding NIBValidator to attribute: %s", [bindingName UTF8String]);
		[component setTarget:[self validator]];
		//[(AZTextField *)component setChangeType:[parser getElementType:bindingName]];
		[(AZTextField *)component setChangeAction:@selector(doProcessControl:)];
	}

	// This code does something if the entity attribute is not optional
/*	NSManagedObject *           managedObject = [[context getController] getObject:nil];
	NSEntityDescription *       entityDesc = [managedObject entity];
	NSDictionary *              entityAttributes = [entityDesc attributesByName];
	if(![[entityAttributes objectForKey:bindingName] isOptional]) {
		[component setBackgroundColor:[NSColor colorForControlTint:NSBlueControlTint]];
	}
*/
}


- (void)processDatePickerDecorations:(id)component forContext:(NIBContext *)context forBinding:(NSString *)bindingName withParent:(NSView *)parent {
	if (![component isKindOfClass:[NSDatePicker class]])
		return;
	
	[(NSDatePicker*)component setDateValue:[NSDate date]];
}


- (void)processPopUpDecorations:(id)component forContext:(NIBContext *)context forBinding:(NSString *)bindingName withParent:(NSView *)parent {
	// Check if this is a dynamic dropdown
	NSString *drop = [[context getMetadata] getValue:bindingName];
	if ([drop hasPrefix:@"<DYNAMIC_"]) {
		NSLog(@"    - building dynamic lookup for: %s", [drop UTF8String]);
		
		NSString *parentWidgetName = [drop substringFromIndex:9];
		parentWidgetName = [parentWidgetName substringToIndex:[parentWidgetName length]-1];
		id parentWidget = [NIBManager getComponentForBinding:parentWidgetName fromParent:parent];
		if (parentWidget && [parentWidget isKindOfClass:[NSPopUpButton class]]) {
			NSLog(@"        - found parent widget %s", [[parentWidget description] UTF8String]);
			
			[parentWidget setTarget:[self handler]];
			[parentWidget setAction:@selector(onPopUpChange:)];
			[[self handler] addDynamicPopUp:bindingName andContextName:context forPopUp:parentWidgetName];
			[[self handler] onPopUpChange:parentWidget];
		}
	}
}


- (void)processValidationDecorations:(id)component forContext:(NIBContext *)context forBinding:(NSString *)bindingName withParent:(NSView *)parent {
	// decorate with validation delegate if component:
	// - has a validator specified (in validations.strings)
	// - accepts a delegate
	if ([[DecoratorParser getDecoratorParser] hasValidator:bindingName forContext:context] && [component respondsToSelector:@selector(setDelegate:)]) {
		if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
			NSLog(@"    - adding validation delegate...");
		[component setDelegate:context];
	}
	
	// decorate with list values if component:
	// - has list values manager specified
	// - is a popup or combo
	if ([[DecoratorParser getDecoratorParser] hasListValues:bindingName forContext:context]) {
		// remove existing values
		[component removeAllItems];
		// add items as appropriate
		if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
			NSLog(@"    - repopulating component with retrieved list values...");
		if ([component isKindOfClass:[NSPopUpButton class]]) {
			[(NSPopUpButton *) component addItemsWithTitles:[[DecoratorParser getDecoratorParser] retrieveListValues:bindingName forContext:context]];
		} else if ([component isKindOfClass:[NSComboBox class]]) {
			[(NSComboBox *) component addItemsWithObjectValues:[[DecoratorParser getDecoratorParser] retrieveListValues:bindingName forContext:context]];
		}
	}
}


- (void)processTokenDecorations:(id)component forContext:(NIBContext *)context forBinding:(NSString *)bindingName withParent:(NSView *)parent {
	if (![component isKindOfClass:[NSTokenField class]])
		return;
	if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
		NSLog(@"        - setting delegate for NSTokenField: %s", [bindingName UTF8String]);

	// Look for max length in metadata
	NSString *			maxValue = [[context getMetadata] getValue:[NSString stringWithFormat:@"%s.max_value", [bindingName UTF8String]]];
	NSManagedObject *	managedO = [[context getController] getObject:nil];
	
	// Create Token Handler
	TokenFieldDelegate *tokenHandler = [[TokenFieldDelegate alloc] init];
	[tokenHandler setManagedObject:managedO];
	[tokenHandler setKey:bindingName];
	if (maxValue)
		[tokenHandler setMaxLength:[maxValue intValue]];
	
	// Set the token handler as the NSTokenField's delegate
	[component setDelegate:tokenHandler];
	
	// Bind the control to the token handler
	[component bind:@"value" toObject:tokenHandler withKeyPath:bindingName options:[component infoForBinding:@"value"]];
	
	// Add the handler as an observer to the backing object
	[managedO addObserver:tokenHandler
			  forKeyPath:bindingName
			  options:(NSKeyValueObservingOptionNew | NSKeyValueObservingOptionOld)
			  context:component];
}


- (void)processOutlineDecorations:(id)component forContext:(NIBContext *)context forBinding:(NSString *)bindingName withParent:(NSView *)parent {
	if (![component isKindOfClass:[NSOutlineView class]])
		return;
	if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
		NSLog(@"        - setting delegate for NSOutlineView: %s", [bindingName UTF8String]);
		
	MetadataParser *				parser = [context getMetadata];
	NSString *						type = [parser getValue:NIB_TYPE];

	if ([type isEqual:TYPE_HIERARCHY]) {
		NSString *					location = [OutlineViewController determinePath:context];
		OutlineViewManager *		manager = [[[OutlineViewManager alloc] init] retain];
		OutlineViewDelegate *		delegate = [[[OutlineViewDelegate alloc] init] retain];
		NSManagedObject *			managedO = [[context getController] getObject:nil];
		NSArray *					mappings = [[[parser getLocalizedValue:[MetadataParser createKey:NIB_OUTLINE withOption:@"mappings"]] componentsSeparatedByString:@","] retain];
		
		if (location) {
			[context setOutlineManager:manager];
			[manager setLabels:[[[parser getLocalizedValue:[MetadataParser createKey:NIB_OUTLINE withOption:@"labels"]] componentsSeparatedByString:@","] retain]];
			[manager setIdentifiers:[[[parser getLocalizedValue:[MetadataParser createKey:NIB_OUTLINE withOption:@"identifiers"]] componentsSeparatedByString:@","] retain]];
			[manager doInitialise:component withURL:[NSURL fileURLWithPath:location]];
			[delegate setManager:manager];
			[delegate setManagedObject:managedO];
			[delegate setMappings:mappings];
			[component setDelegate:delegate];
			[component deselectAll:manager];
		}
	}
}


- (IBAction)doPopUpChange:(id)sender {
    if (![self isEnabled])
        return;
    
    NSString *          selected = [popupMenu titleOfSelectedItem];
    NIBContext *		context;
    int                 i;

    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
        NSLog(@"NIBManager received change event: %s", [selected UTF8String]);
    for (i = 0; i < [self getContextCount]; i++) {
        context = [self getContext:i];
        if ([selected isEqualTo:[[context getMetadata] getLabel]]) {
            if (tabView) {
                [tabView selectTabViewItemWithIdentifier:[[context getMetadata] getName]];
            } else {
                [[context getWindow] makeKeyAndOrderFront:sender];
			}
        }
    }
	if ([[self main] respondsToSelector:@selector(doPopUpChange:)])
		[[self main] doPopUpChange:sender];
}


- (IBAction)onPopUpChange:(id)sender {
    [self performSelectorOnMainThread:@selector(doPopUpChange:)
                           withObject:sender
                        waitUntilDone:YES];
}


- (void)doDisplay:(NSString *)type {
	[self doDisplay:type withIdentifier:nil];
}


- (void)doDisplay:(NSString *)type withIdentifier:(id)identifier {
    NIBContext *		context;
	NSString *			temp;
	int					i;

    for (i = 0; i < [self getContextCount]; i++) {
        context = [self getContext:i];
		temp = [[context getMetadata] getName];
        if ([type isEqualTo:temp]) {
            if (tabView) {
                [tabView selectTabViewItemWithIdentifier:temp];
            } else {
                [[context getWindow] makeKeyAndOrderFront:self];
			}
        }
    }
}


- (IBAction)doExecute:(id)sender {
    NIBController *     controller;
    int                 i;
    
    if (![self isEnabled])
        return;
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
        NSLog(@"NIBManager received execute event...");
    for (i = 0; i < [self getControllerCount]; i++) {
        controller = [self getController: i];
        if ([controller isEnabled])
            [controller doExecute: sender];
    }
}


- (void)doCallback:(NIBController *)controller withObject:(id)object {
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
        NSLog(@"NIBManager received callback event...");
    if (callback)
        [callback doCallback:controller withObject:object];
}


- (void)doBuild:(NSPopUpButton *)popup {
	[self doBuild:popup withTarget:self withAction:@selector(onPopUpChange:)];
}


- (void)doBuild:(NSPopUpButton *)popup withTarget:(id)target withAction:(SEL)action {
	MetadataParser *	parser;
    NSString *          title;
    int                 i;
	int					total = [popup numberOfItems];
	int					count = 0;
    
    for (i = 0; i < [self getContextCount]; i++) {
		parser = [[self getContext:i] getMetadata];
        title = [parser getLabel];
		if (title && [title length] > 0) {
			[parser setTag:(total+count)];
            [popup addItemWithTitle:title];
			[[[popup menu] itemWithTitle:title] setTag:[parser getTag]];
			count++;
			if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
				NSLog(@"NIBManager added option to NSPopUpButton, title: %s", [title UTF8String]);
		} else
			NSLog(@"NIBManager found ill-formed NIB at index: %i", i);
    }
    [self setPopupMenu:popup withTarget:target withAction:action];
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
        NSLog(@"NIBManager build completed");
}


- (void)doBuild:(NSPopUpButton *)menu withTabs:(NSTabView *)tabs {
	[self doBuild:menu withTabs:tabs withTarget:self withAction:@selector(onPopUpChange:)];
}


- (void)doBuild:(NSPopUpButton *)menu withTabs:(NSTabView *)tabs withTarget:(id)target withAction:(SEL)action {
	NIBContext *		context;
    NSString *          temp;
    NSView *            view;
    int                 i;
    
    for (i = 0; i < [self getContextCount]; i++) {
		context = [self getContext:i];
        view = [context getView];
        if (view) {
            temp = [[context getMetadata] getName];
			if (temp && [temp length] > 0) {
				[NIBManager addComponent:view withTabs:tabs withIdentifier:temp];
				if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
					NSLog(@"NIBManager added view to NSTabView, name: %s", [temp UTF8String]);
			} else
				NSLog(@"NIBManager found ill-formed NIB at index: %i", i);
        }
    }
    [self setTabView:tabs];
	[self doBuild:menu withTarget:target withAction:action];
}


- (void)doBuildSearch:(NSSearchField *)field withTemplate:(NSMenu *)template withSelector:(SEL)aSelector {
	MetadataParser *	parser;
    NSMenuItem *        item;
	NSString *			title;
	int					i;
    
	for (i = 0; i < [self getContextCount]; i++) {
		parser = [[self getContext:i] getMetadata];
		title = [parser getLabelWith:NIB_SEARCH_LABEL];
		if (title && [title length] > 0) {
			item = [[NSMenuItem alloc] initWithTitle:title
											  action:aSelector
									   keyEquivalent:@""];
			[item setTag: [parser getTag]];
			[template addItem:item];
			[item release];
		}
	}
    [[field cell] setSearchMenuTemplate: template];
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_INFO)
        NSLog(@"NIBManager searcher build completed");
}


- (NIBController *)createController {
    return [[[NIBController alloc] init] retain];
}


- (NIBController *)getController:(int)index {
    return (NIBController *) [controllers objectAtIndex: index];
}


- (void)addController:(NIBController *)controller {
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
        NSLog(@"NIBManager adding NIBController...");
    [controllers addObject: controller];
}


- (void)removeController:(NIBController *)controller {
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
        NSLog(@"NIBManager removing NIBController...");
    [controllers removeObject: controller];
}


- (int)getControllerCount {
    return [controllers count];
}


- (int)getContextCount {
	return [contexts count];
}


- (int)determineTag:(NSString *)type {
	MetadataParser *	parser;
	int					i = 0;
	
	for (i = 0; i < [self getContextCount]; i++) {
		parser = [[self getContext:i] getMetadata];
		if ([[parser getName] isEqualTo:type])
			return [parser getTag];
	}
	return -1;
}


- (BOOL)isEnabled {
    return ([contexts count] > 0);
}


- (NIBContext *)getContext:(int)index {
    return (NIBContext *) [contexts objectAtIndex:index];
}


- (NIBContext *)getContextFor:(NSString *)type {
	NIBContext *	context;
	int				tag = [self determineTag:type];
	
    if ([NIBManagerHandler getLoggingLevel] >= LOGGING_DEBUG)
        NSLog(@"NIBManager getting context: %s [%i]", [type UTF8String], tag);
    context = (tag >= 0 ? [self getContext:tag] : nil);
	if ([self persistenceMode] == UNIQUE) {
		context = [self doCreate:[context getPath] withName:[context getName] withTag:[[context getMetadata] getTag]];
	}
	return context;
}


- (NSArray *)topLevelObjectsFromLoadingNibNamed:(NSString *)myName withOwner:(id)owner {
    return [self topLevelObjectsFromLoadingBundle:[NSBundle bundleForClass:[owner class]] 
                                               withName:myName
                                              withOwner:owner];
}


- (NSArray *)topLevelObjectsFromLoadingBundle:(NSBundle *)bundle withName:(NSString *)myName withOwner:(id)owner {
    NSString *            file = nil;
    
    if ([myName hasSuffix:@".bundle"])
        myName = [myName substringToIndex: [myName length]-7];
    if (bundle)
        file = [bundle pathForResource:myName ofType:@"nib"];
    if (!file)
        file = [bundle pathForResource:[self name] ofType:@"nib"];
    if (!file) {
        bundle = [NSBundle mainBundle];
        file = [bundle pathForResource:myName ofType:@"nib"];
    }
    if (!file)
        file = [bundle pathForResource:[self name] ofType:@"nib"];
    if (!file && [NIBManagerHandler getLoggingLevel] >= LOGGING_NOTICE)
        NSLog(@"NIBManager unable to load NIB objects from bundle: %s", [myName UTF8String]);
    return [self topLevelObjectsFromLoadingBundleNib:bundle
											withName:file 
											withOwner:owner];
}


- (NSArray *)topLevelObjectsFromLoadingBundleNib:(NSBundle *)bundle withName:(NSString *)myName withOwner:(id)owner {
    NSMutableArray *        topLevelObjects = [NSMutableArray array];
    NSDictionary *          nameTable = [NSDictionary dictionaryWithObjectsAndKeys:topLevelObjects, @"NSTopLevelObjects", owner, @"NSOwner", nil];
    
    if ([NSBundle loadNibFile:myName 
            externalNameTable:nameTable 
                     withZone:[owner zone]]) {
        [topLevelObjects makeObjectsPerformSelector:@selector(release)];
        return topLevelObjects;
    }
    if (myName && [NIBManagerHandler getLoggingLevel] >= LOGGING_NOTICE)
        NSLog(@"NIBManager unable to load NIB objects from file: %s", [myName UTF8String]);
    return nil;
}


/**
 * Maps the given array into a new array of NSMutableDictionary(s) ready for display in a
 * NSTableView backed by an NSArrayController. Values for display are identified by the
 * metadata specified for the object type.  Given results array is added to the new
 * dictionary (key = "data") to enable access to non-displayed metadata values.
 *
 * The structure of a sample dictionary is, for example:
 *      identifier = "12345";
 *      title = "Title";
 *      description = "Description";
 *      submitter = "Submitter";
 *      date = "2006/06/16";
 *      data = <resultsArray>;
 */
- (NSArray *)createResultsArrayForDisplay:(NSArray *)resultsArray {
    // sanity check
    if (!resultsArray || [resultsArray count] == 0) {
        return [[NSArray alloc] init];
    }
    
    // retrieve metadata for objects in results array
    // - NOTE: assumes all objects in results is the same object type, hence uses the same metadata
    NSString *              objectType = [[resultsArray objectAtIndex:0] valueForKey:DEFAULT_RESULT_TYPE_KEY];
    int                     tag = [self determineTag:objectType];
    MetadataParser *        metadata = [[self getContext:tag] getMetadata];
    
    // retrieve result table column identifiers from the metadata
    NSArray *               resultIdentifiers = [metadata searchResultIdentifiers];
    
    // create output array
    NSMutableArray *        results = [[NSMutableArray alloc] init];
    
    int                     i, j;
    NSDictionary *          dictionary; // individual dictionary passed in
    NSMutableDictionary *   newDictionary; // new dictionary to form part of results
    
    // transform each dictionary passed in to a dictionary with only the key-value pairs required for result display (specified by the metadata)
    NSString *              resultIdentifier; 
    for (i = 0; i < [resultsArray count]; i++) {
        dictionary = [resultsArray objectAtIndex:i];
        newDictionary = [[NSMutableDictionary alloc] init];
        // add key-value pair into result dictionary
        for (j = 0; j < [resultIdentifiers count]; j++) {
            resultIdentifier = [resultIdentifiers objectAtIndex:j];
			if(![dictionary objectForKey:[metadata searchResultMetadataName:resultIdentifier]]) {
				[newDictionary setObject:@"" forKey:resultIdentifier];
			}
			else {
				[newDictionary setObject:[dictionary objectForKey:[metadata searchResultMetadataName:resultIdentifier]] forKey:resultIdentifier];
			}
        }
        // add the original dictionary as a value (used later for editing remaining results)
        [newDictionary setObject:dictionary forKey:DEFAULT_RESULT_DATA_KEY];
		
		[results addObject:newDictionary];
		
    }
    return results;    
}


/**
 * Maps the given array into a new array of NSMutableDictionary(s) ready for display in a
 * NSTableView backed by an NSArrayController. Values for display are identified by the
 * metadata specified for the object type.  Given results array is added to the new
 * dictionary (key = "data") to enable access to non-displayed metadata values.
 *
 * The structure of a sample dictionary is, for example:
 *      identifier = "12345";
 *      title = "Title";
 *      description = "Description";
 *      submitter = "Submitter";
 *      date = "2006/06/16";
 *      data = <resultsArray>;
 */
- (NSDictionary *)createResultsDictionaryForDisplay:(NSDictionary *)results {

    // retrieve metadata for objects in results array
    // - NOTE: assumes all objects in results is the same object type, hence uses the same metadata
    NSString *              objectType = [results valueForKey:DEFAULT_RESULT_TYPE_KEY];
    int                     tag = [self determineTag:objectType];
    MetadataParser *        metadata = [[self getContext:tag] getMetadata];
    NSMutableDictionary *   newDictionary = [[[NSMutableDictionary alloc] init] retain]; // new dictionary to form part of results
    
    // retrieve result table column identifiers from the metadata
    NSArray *               resultIdentifiers = [metadata searchResultIdentifiers];
    NSString *              resultIdentifier; 
    int                     i;

	// add key-value pair into result dictionary
	for (i = 0; i < [resultIdentifiers count]; i++) {
		resultIdentifier = [resultIdentifiers objectAtIndex:i];
		if (![results objectForKey:[metadata searchResultMetadataName:resultIdentifier]]) {
			[newDictionary setObject:@"" forKey:resultIdentifier];
		} else {
			[newDictionary setObject:[results objectForKey:[metadata searchResultMetadataName:resultIdentifier]] forKey:resultIdentifier];
        }
	}
	// add the original dictionary as a value (used later for editing remaining results)
	[newDictionary setObject:results forKey:DEFAULT_RESULT_DATA_KEY];
    return newDictionary;    
}


+(void)addComponent:(NSView *)component withTabs:(NSTabView *)tabs withIdentifier:(NSString *)identifier {
	NSTabViewItem *		item = [[NSTabViewItem alloc] initWithIdentifier:identifier];

	[item setLabel:identifier];
	[tabs addTabViewItem:item];
	[NIBManager addComponent:component withTabs:tabs withItem:item];
}


+(void)addComponent:(NSView *)component withTabs:(NSTabView *)tabs withItem:(NSTabViewItem *)item {
	component = (NSView *) [NIBManager getContentComponent:component];
	[NIBManager originateComponent:component];
	[item setView:component];
}


+(void)addAction:(NSView *)view withTarget:(id)target withAction:(SEL)action {
	[NIBManager addAction:view withDelegate:nil withTarget:target withAction:action];
}


+(void)addAction:(NSView *)view withDelegate:(id)delegate withTarget:(id)target withAction:(SEL)action {
	NSArray *		views = [view subviews];
	id				temp;
	int				i;
	
	for (i = 0; i < [views count]; i++) {
		temp = [views objectAtIndex:i];
		if ([temp isKindOfClass:[NSTextField class]]) {
			if (delegate) {
				[temp setDelegate:delegate];
			}
		} else if ([temp isKindOfClass:[NSButton class]]) {
			if (![temp target] || ![temp action]) {
				[temp setTarget:target];
				[temp setAction:action];
			}
		} else if ([temp isKindOfClass:[NSView class]]) {
			[NIBManager addAction:(NSView *)temp withDelegate:delegate withTarget:target withAction:action];
		}
	}
}


+(void)removeComponent:(NSView *)view {
	[NIBManager removeComponent:view fromParent:[view superview]];
}


+(void)removeComponent:(NSView *)view fromParent:(NSView *)parent {
	NSView * tempView = [[NSView alloc] initWithFrame:NSZeroRect];

	[parent replaceSubview:view with:tempView];
	[tempView release];
}


+(void)originateComponent:(NSView *)component {
	NSPoint				point = {0, 0};

	if ([component isKindOfClass:[NSScrollView class]])
		[[(NSScrollView *)component documentView] scrollPoint:point];
	else
		[component scrollPoint:point];
}


+(NSView *)makeFlippedView:(NSView *)component {
	return [NIBManager makeFlippedView:component withAddition:YES];
}


+(NSView *)makeFlippedView:(NSView *)component withAddition:(BOOL)addition {
	if ([component isKindOfClass:[AZView class]])
		return component;
		
	AZView *		flipped = [[[AZView alloc] init] retain];
			
	[flipped setMenu:[component menu]];
	[flipped setFrame:[component frame]];
	[flipped setBounds:[component bounds]];
	if (addition)
		[flipped addSubview:component];
	return flipped;
}


+(NSBox *)makeBoxedView:(NSView *)component withTitle:(NSString *)title {
	return [NIBManager makeBoxedView:component withTitle:title withFlipped:YES];
}


+(NSBox *)makeBoxedView:(NSView *)component withTitle:(NSString *)title withFlipped:(BOOL)flipped {
	NSBox *			boxed = [[[NSBox alloc] init] retain];
			
	if (flipped)
		[boxed setContentView:[NIBManager makeFlippedView:component withAddition:NO]];
	else
		[[boxed contentView] setMenu:[component menu]];
	[boxed setMenu:[component menu]];
	[boxed setTitle:title];
	[boxed setAutoresizingMask:[component autoresizingMask]];
	return boxed;
}


+(NSScrollView *)makeScrollView:(NSView *)component withFlipped:(BOOL)flipped {
	return [NIBManager makeScrollView:component withBounds:[component bounds] withFlipped:flipped];
}


+(NSScrollView *)makeScrollView:(NSView *)component withBounds:(NSRect)bounds withFlipped:(BOOL)flipped {
	NSScrollView *	scroller = [[[NSScrollView alloc] init] retain];
			
	[scroller setAutohidesScrollers:YES];
	[scroller setHasVerticalScroller:YES];
	[scroller setHasHorizontalScroller:YES];
	[scroller setDocumentView:(flipped ? [NIBManager makeFlippedView:component] : component)];
	[scroller setDrawsBackground:NO];
	[scroller setFrame:bounds];
	[scroller setBounds:bounds];
	[scroller setMenu:[component menu]];
	[component scrollPoint:bounds.origin];
	return scroller;
}


+(NSWindow *)makeWindow:(NSView *)component withTitle:(NSString *)title {
	return [NIBManager makeWindow:component withTitle:title withStyle:(NSClosableWindowMask | NSMiniaturizableWindowMask | NSResizableWindowMask)];
}


+(NSWindow *)makeWindow:(NSView *)component withTitle:(NSString *)title withStyle:(unsigned int)style {
	NSWindow *		window;
	
	if (title)
		style = (style | NSTitledWindowMask);
	window = [[NSWindow alloc] initWithContentRect:[component bounds] styleMask:style backing:NSBackingStoreBuffered defer:NO];
	if (title)
		[window setTitle:title];
	[window setContentView:component];
	[window setContentSize:[component bounds].size];
	return window;
}


+(NSPanel *)makePanel:(NSView *)component withTitle:(NSString *)title {
	return [NIBManager makePanel:component withTitle:title withStyle:(NSClosableWindowMask)];
}


+(NSPanel *)makePanel:(NSView *)component withTitle:(NSString *)title withStyle:(unsigned int)style {
	NSPanel *		window;
	
	if (title)
		style = (style | NSTitledWindowMask);
	window = [[NSPanel alloc] initWithContentRect:[component bounds] styleMask:style backing:NSBackingStoreBuffered defer:NO];
	if (title)
		[window setTitle:title];
	[window setContentView:component];
	return window;
}


+(NSWindow *)makeWindow:(NIBContext *)context withView:(NSView *)view  withTitle:(NSString *)title {
	return [NIBManager makeWindow:context withView:view withTitle:title withStyle:(NSClosableWindowMask | NSMiniaturizableWindowMask | NSResizableWindowMask)];
}


+(NSWindow *)makeWindow:(NIBContext *)context withView:(NSView *)view  withTitle:(NSString *)title withStyle:(unsigned int)style {
	NSWindow *		window = [NIBManager makeWindow:view withTitle:title withStyle:style];
	double			width = [[context getMetadata] getWidth];
	double			height = [[context getMetadata] getHeight];
	
	if (width > 0 && height > 0) {
		NSSize		size = {width, height};
		[window setContentSize:size];
	}
	return window;
}


+(NSPanel *)makePanel:(NIBContext *)context withView:(NSView *)view  withTitle:(NSString *)title {
	return [NIBManager makePanel:context withView:view withTitle:title withStyle:(NSClosableWindowMask)];
}


+(NSPanel *)makePanel:(NIBContext *)context withView:(NSView *)view  withTitle:(NSString *)title withStyle:(unsigned int)style {
	NSPanel *		window = [NIBManager makePanel:view withTitle:title withStyle:style];
	double			width = [[context getMetadata] getWidth];
	double			height = [[context getMetadata] getHeight];
	
	if (width > 0 && height > 0) {
		NSSize		size = {width, height};
		[window setContentSize:size];
	}
	return window;
}


+(BOOL)makeDirectoryPath:(NSString *)path {
    NSFileManager *                 fileManager = [NSFileManager defaultManager];
    BOOL                            isDirectory = NO;

	path = [path stringByExpandingTildeInPath];
	if ([fileManager fileExistsAtPath:path isDirectory:&isDirectory])
		return isDirectory;

	NSMutableArray *				components = [NSMutableArray arrayWithArray:[path pathComponents]];
	NSString *						pathElement;
	NSString *						dir = @"";
	int								i;
    
	for (i = 0 ; i < [components count]; i++) {
		pathElement = [components objectAtIndex:i];
		dir = [dir stringByAppendingPathComponent:pathElement];
		if (![fileManager fileExistsAtPath:dir isDirectory:&isDirectory]) {
			if (![fileManager createDirectoryAtPath:dir attributes:nil]) {
				[dir release];
				dir = nil;
				return NO;
			}
		}
	}
	return YES;
}


+(void)setEnabled:(BOOL)enabled withView:(NSView *)view {
	[NIBManager setEnabled:enabled withView:view withFilter:nil];
}


+(void)setEnabled:(BOOL)enabled withView:(NSView *)view withFilter:(NSString *)filter {
	NSArray *		views = [view subviews];
	BOOL			matched;
	id				temp;
	int				i;
	
	for (i = 0; i < [views count]; i++) {
		temp = [views objectAtIndex:i];
		matched = (!filter || [filter isEqual:[NIBManager determineEnabled:temp]]);
		if ([temp isKindOfClass:[NSControl class]]) {
			if (matched)
				[temp setEnabled:enabled];
		} else if ([temp isKindOfClass:[NSTextView class]]) {
			if (matched)
				[temp setEditable:enabled];
		} else if ([temp isKindOfClass:[NSScrollView class]]) {
			[NIBManager setEnabled:enabled withView:[temp documentView] withFilter:(matched ? nil : filter)];
		} else if ([temp isKindOfClass:[NSBox class]]) {
			[NIBManager setEnabled:enabled withView:[temp contentView] withFilter:(matched ? nil : filter)];
		} else if ([temp isKindOfClass:[NSTabView class]]) {
			[NIBManager setEnabled:enabled withView:[[temp selectedTabViewItem] view] withFilter:(matched ? nil : filter)];
		} else if ([temp isKindOfClass:[NSView class]]) {
			[NIBManager setEnabled:enabled withView:temp withFilter:(matched ? nil : filter)];
		}
	}
}


+(void)setValue:(id)value withKey:(NSString *)key withParent:(NSView *)view {
	NSArray *		views = [view subviews];
	id				temp;
	int				i;
	
	for (i = 0; i < [views count]; i++) {
		temp = [views objectAtIndex:i];
		if ([temp isKindOfClass:[NSControl class]]) {
			if ([[NIBManager determineName:temp] isEqualTo:key])
				[temp setStringValue:value];
		} else if ([temp isKindOfClass:[NSView class]]) {
			[NIBManager setValue:value withKey:key withParent:temp];
		}
	}
}


+(NSMutableArray *)getComponents:(id)parent {
	NSMutableArray *	components = [[[NSMutableArray alloc] init] retain];
	NSArray *			subviews = nil;
	int					i = 0;
	
	if ([parent isKindOfClass:[NSWindow class]]) {
		[components addObjectsFromArray:[NIBManager getComponents:[parent contentView]]];
	} else if ([parent isKindOfClass:[NSBox class]]) {
		[components addObjectsFromArray:[NIBManager getComponents:[parent contentView]]];
	} else if ([parent isKindOfClass:[NSScrollView class]]) {
		[components addObjectsFromArray:[NIBManager getComponents:[parent documentView]]];
	} else if ([parent isKindOfClass:[NSTabViewItem class]]) {
		[components addObjectsFromArray:[NIBManager getComponents:[parent view]]];

	} else if ([parent isKindOfClass:[NSControl class]]) {
		// If there is a makeKeyAndOrderFront: connection on this NSControl and the target
		// is an NSPanel, get the components in that panel too
		id	target = [parent target];
		if ([target isKindOfClass:[NSPanel class]] && [parent action] == @selector(makeKeyAndOrderFront:)) {
			[components addObjectsFromArray:[NIBManager getComponents:[target contentView]]];
		}
		[components addObject:parent];
	} else if ([parent isKindOfClass:[NSTextView class]]) {
		[components addObject:parent];
	} else if ([parent isKindOfClass:[NSTabView class]]) {
		subviews = [parent tabViewItems];
	} else if ([parent isKindOfClass:[NSView class]]) {
		//[components addObject:parent];
		subviews = [parent subviews];
	}
		
	for (i = 0; subviews && i < [subviews count]; i++)
		[components addObjectsFromArray:[NIBManager getComponents:[subviews objectAtIndex:i]]];
	return components;
}


+(id)getComponent:(NSString *)name fromParent:(id)component {
	if ([name isEqualTo:[NIBManager determineIdentifier:component]])
		return component;
	
	NSArray *		subviews = [NIBManager getComponents:component];
	NSString *		temp;
	int				i = 0;
	id				child;
	
	for (i = 0; i < [subviews count]; i++) {
		child = [subviews objectAtIndex:i];
		temp = [NIBManager determineIdentifier:child];
		if (temp && [temp isEqualTo:name])
			return child;
	}
	return nil;
}


+(id)getComponentOrView:(NSString *)name fromParent:(id)component {
	if ([name isEqualTo:[NIBManager determineIdentifier:component]])
		return component;
	
	NSArray *		subviews = [component subviews];
	int				i = 0;
	id				child;
	
	for (i = 0; i < [subviews count]; i++) {
		child = [NIBManager getComponentOrView:name fromParent:[subviews objectAtIndex:i]];
		if (child)
			return child;
	}
	return nil;
}


+(id)getComponentForBinding:(NSString *)bindingName fromParent:(id)component {
	NSArray *		subviews = [NIBManager getComponents:component];
	NSString *		temp;
	int				i = 0;
	id				child;
	
	for (i = 0; i < [subviews count]; i++) {
		child = [subviews objectAtIndex:i];
		temp = [NIBManager determineName:child];
		if (temp && [temp isEqualTo:bindingName])
			return child;
	}
	return nil;
}


+(id)getContentComponent:(id)component {
	if (![component isKindOfClass:[NSScrollView class]])
		return component;

	id				content = [(NSScrollView *)component documentView];
	
	if ([content isKindOfClass:[AZView class]] && [[content subviews] count] == 1)
		content = [[content subviews] objectAtIndex:0];
	if ([content isKindOfClass:[NSView class]] && [[content subviews] count] == 1)
		content = [[content subviews] objectAtIndex:0];
	//if ([content isKindOfClass:[NSScrollView class]] || [content isKindOfClass:[SM2DGraphView class]] || [content isKindOfClass:[SMPieChartView class]])
	if ([content isKindOfClass:[NSScrollView class]])
		return content;
	return component;
}


+(id)determineComponent:(id)component {
	if (!component)
        NSLog(@"NIBManager considering NULL component...");

	if ([component isKindOfClass:[MatrixController class]]) {
		return [NIBManager determineComponent:[component context]];
	} else if ([component isKindOfClass:[NIBContext class]]) {
		return [NIBManager determineComponent:[component getView]];
	} else if ([component isKindOfClass:[NIBController class]]) {
		return [NIBManager determineComponent:[component window]];
	} else if ([component isKindOfClass:[NSWindow class]]) {
		return [NIBManager determineComponent:[component contentView]];
	} else if ([component isKindOfClass:[NSView class]]) {
		return component;
	} else {
		return nil;
	}
}


+(NSString *)determineName:(id)component {
	NSString *			value = nil;
	
	if ([component isKindOfClass:[NSButton class]] && ![component isKindOfClass:[NSPopUpButton class]] && ![component isKindOfClass:[NSComboBox class]])
		return [component alternateTitle];
	if ([component isKindOfClass:[NSTextView class]])
		return [NIBManager determineBinding:@"data" withComponent:component];
	value = [NIBManager determineBinding:@"value" withComponent:component];
	if (!value)
		value = [NIBManager determineBinding:@"selectedValue" withComponent:component];
	return value;
}


+(NSString *)determineIdentifier:(id)component {
	if ([component isKindOfClass:[NSMenu class]] && [[component delegate] isKindOfClass:[NSObjectController class]])
		return [[component delegate] entityName];
	return [NIBManager determineBinding:@"toolTip" withComponent:component withOffset:7];
}


+(NSString *)determineEnabled:(id)component {
	NSString *			value = [NIBManager determineName:component];
	return (value ? value : [NIBManager determineBinding:@"enabled" withComponent:component withOffset:7]);
}


+(NSString *)determineBinding:(NSString *)key withComponent:(id)component {
	return [NIBManager determineBinding:key withComponent:component withOffset:10];
}


+(NSString *)determineBinding:(NSString *)key withComponent:(id)component withOffset:(int)offset {
	NSString *			value = [[component infoForBinding:key] objectForKey:NSObservedKeyPathKey];
	return (value ? [value substringFromIndex:offset] : nil);
}


@end
