//
//  NIBManagerHandler.m
//  DynamicNIB
//
//  Copyright 2005 __MyCompanyName__. All rights reserved.
//

#ifndef INCLUDED_NIBMANAGERHANDLER_H
#import "NIBManagerHandler.h"
#endif
#ifndef INCLUDED_NIBMANAGER_H
#import "NIBManager.h"
#endif
#ifndef INCLUDED_NIBCALLBACK_H
#import "NIBCallback.h"
#endif
#ifndef INCLUDED_NIBCONTEXT_H
#import "NIBContext.h"
#endif



NSString*		LOCALE_ENGLISH		= @"en";
NSString*		LOCALE_JAPANESE		= @"jp";
NSString*		KEY_IDENTIFIER		= @"key.identifier";

int             MANAGER_STAFFWARE   = 1;
int             MANAGER_DOCUMENTUM  = 2;
int             MANAGER_PENTAHO		= 3;
int             MANAGER_MAINFRAME	= 4;
int             LOGGING_NONE        = 0;
int             LOGGING_NOTICE      = 1;
int             LOGGING_INFO        = 2;
int             LOGGING_DEBUG       = 3;
int             EMBEDDED			= 0;
int             WINDOWED			= 1;
int             DOCUMENT			= 0;
int             UNIQUE				= 1;


static NIBManagerHandler * __THIS   = nil;
static int                 __LEVEL  = 0;


@implementation NIBManagerHandler


- (id)init {
    self = [super init];
    if (self) {
        __THIS = self;
        if (__LEVEL == LOGGING_NONE)
            __LEVEL = LOGGING_DEBUG;
        managers = [[NSMutableDictionary dictionary] retain];
		if ([NIBManagerHandler getLoggingLevel] >= LOGGING_NOTICE)
			NSLog(@"DynamicNIB.framework bundle version: %s", [[NIBManagerHandler getBundleVersion] UTF8String]);
    }
    return self;
}


- (void)dealloc {
    if (managers)
        [managers release];
    [super dealloc];
}



-(void)awakeFromNib {
    __THIS = self;
}


+(NSString *)getBundleVersion {
    return [[[NSBundle bundleForClass:[NIBManagerHandler class]] infoDictionary] objectForKey:@"CFBundleVersion"];
}


+(NSString *)getLocaleIdentifier {
	NSUserDefaults*		myDefaults = [NSUserDefaults standardUserDefaults];
	NSArray*			myLanguages = [myDefaults objectForKey:@"AppleLanguages"];

	// Hack to convert the locale identifier of 'ja' to 'jp'
	if ([[myLanguages objectAtIndex:0] isEqualToString:@"ja"]) {
		return LOCALE_JAPANESE;
	}
	return [myLanguages objectAtIndex:0];
}


+(BOOL)isEnglish {
    return [[NIBManagerHandler getLocaleIdentifier] isEqualTo:LOCALE_ENGLISH];
}


+(BOOL)isJapanese {
    return [[NIBManagerHandler getLocaleIdentifier] isEqualTo:LOCALE_JAPANESE];
}


+(NIBManagerHandler *)getHandler {
    if (__THIS)
        return __THIS;
    return (__THIS = [[[NIBManagerHandler alloc] init] retain]);
}


+(int)getLoggingLevel {
    return __LEVEL;
}


+(void)setLoggingLevel:(int)level {
	if (level == LOGGING_NONE)
		NSLog(@"DynamicNIB.framework assigning logging level: NONE");
	if (level == LOGGING_NOTICE)
		NSLog(@"DynamicNIB.framework assigning logging level: NOTICE");
	if (level == LOGGING_INFO)
		NSLog(@"DynamicNIB.framework assigning logging level: INFO");
	if (level == LOGGING_DEBUG)
		NSLog(@"DynamicNIB.framework assigning logging level: DEBUG");
    __LEVEL = level;
}


-(NIBContext *)getContext:(NSString *)type withIndex:(int)index {
	return [[self getManager:type] getContext:index];
}


-(NIBManager *)getManager:(NSString *)type {
	return [managers objectForKey:type];
}


-(void)setManager:(NIBManager *)manager withType:(NSString *)type {
	[managers setValue:manager forKey:type];
}


- (void)setBackingType:(NSString *)myType {
	NSArray *		types = [managers allKeys];
	int				i ;
	
	for (i = 0; i < [types count]; i++)
		[[self getManager:[types objectAtIndex:i]] setBackingType:myType];
}


- (void)setBackingStore:(NSString *)myStore {
	NSArray *		types = [managers allKeys];
	int				i ;
	
	for (i = 0; i < [types count]; i++)
		[[self getManager:[types objectAtIndex:i]] setBackingStore:myStore];
}


- (void)setScrolling:(BOOL)isScrolling {
	NSArray *		types = [managers allKeys];
	int				i ;
	
	for (i = 0; i < [types count]; i++)
		[[self getManager:[types objectAtIndex:i]] setScrolling:isScrolling];
}


- (void)setFlipped:(BOOL)isFlipped {
	NSArray *		types = [managers allKeys];
	int				i ;
	
	for (i = 0; i < [types count]; i++)
		[[self getManager:[types objectAtIndex:i]] setFlipped:isFlipped];
}


- (void)setDecorated:(BOOL)isDecorated {
	NSArray *		types = [managers allKeys];
	int				i ;
	
	for (i = 0; i < [types count]; i++)
		[[self getManager:[types objectAtIndex:i]] setDecorated:isDecorated];
}


- (void)setBounds:(NSRect)myBounds {
	NSArray *		types = [managers allKeys];
	int				i ;
	
	for (i = 0; i < [types count]; i++)
		[[self getManager:[types objectAtIndex:i]] setBounds:myBounds];
}


- (void)setAdjustment:(NSPoint)myAdjustment {
	NSArray *		types = [managers allKeys];
	int				i ;
	
	for (i = 0; i < [types count]; i++)
		[[self getManager:[types objectAtIndex:i]] setAdjustment:myAdjustment];
}


-(NIBManager *)doRegister:(NSString *)manager withType:(int)type {
	return [self doRegister:manager withType:type withPersistence:DOCUMENT];
}
	
	
-(NIBManager *)doRegister:(NSString *)identifier withType:(int)type withPersistence:(int)persistence {
	NIBManager *	manager;
	
	if ([NIBManagerHandler getLoggingLevel] >= LOGGING_NOTICE)
		NSLog(@"DynamicNIB.framework creating/registering NIBManager: %s", [identifier UTF8String]);
     if (type == MANAGER_STAFFWARE) {
		manager = [[[NIBManager alloc] init] retain];
	} else if (type == MANAGER_DOCUMENTUM) {
		manager = [[[NIBManager alloc] init] retain];
	} else if (type == MANAGER_MAINFRAME) {
		manager = [[[NIBManager alloc] init] retain];
	} else if (type == MANAGER_PENTAHO) {
		manager = [[[NIBManager alloc] init] retain];
	}
	[manager setPersistenceMode:persistence];
	if (persistence != DOCUMENT) {
		[managers setValue:manager forKey:identifier];
	}
	return manager;
}


- (void)doInitialise:(NSString *)type withPath:(NSString *)myPath {
	[self doInitialise:type withPath:myPath withCallback:nil];
}


- (void)doInitialise:(NSString *)type withPath:(NSString *)myPath withCallback:(NIBCallback *)myCallback {
	[self doInitialise:type withPath:myPath withCallback:myCallback withValidator:nil];
}


- (void)doInitialise:(NSString *)type withPath:(NSString *)myPath withCallback:(NIBCallback *)myCallback withValidator:(NIBValidator *)myValidator {
    NIBManager *      manager = [self getManager:type];
    
    if (manager)
        [manager doInitialise:myPath withCallback:myCallback withValidator:myValidator];
}


- (int)determineTag:(NSString *)type {
	NSArray *		types = [managers allKeys];
	int				tag = -1;
	int				i ;
	
	for (i = 0; i < [types count]; i++) {
		tag = [[self getManager:[types objectAtIndex:i]] determineTag:type];
		if (tag >= 0)
			return tag;
	}
	return tag;
}


@end
